﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.TestFunctions {
  /// <summary>
  /// Base class for a test function evaluator.
  /// </summary>
  [Item("Single-Objective Function", "Base class for single objective functions.")]
  [StorableClass]
  public abstract class MultiObjectiveTestFunction : ParameterizedNamedItem, IMultiObjectiveTestFunction {
    /// <summary>
    /// These operators should not change their name through the GUI
    /// </summary>
    public override bool CanChangeName {
      get { return false; }
    }
    /// <summary>
    /// Returns whether the actual function constitutes a maximization or minimization problem.
    /// </summary>
    public abstract bool[] Maximization { get; }
    /// <summary>
    /// Gets the lower and upper bound of the function.
    /// </summary>
    public abstract DoubleMatrix Bounds { get; }
    /// <summary>
    /// Gets the minimum problem size.
    /// </summary>
    public abstract int MinimumProblemSize { get; }
    /// <summary>
    /// Gets the maximum problem size.
    /// </summary>
    public abstract int MaximumProblemSize { get; }
    /// <summary>
    /// Gets the minimum problem size.
    /// </summary>
    public abstract int MinimumSolutionSize { get; }
    /// <summary>
    /// Gets the maximum problem size.
    /// </summary>
    public abstract int MaximumSolutionSize { get; }

    [StorableConstructor]
    protected MultiObjectiveTestFunction(bool deserializing) : base(deserializing) { }
    protected MultiObjectiveTestFunction(MultiObjectiveTestFunction original, Cloner cloner) : base(original, cloner) { }
    protected MultiObjectiveTestFunction() : base() { }


    /// <summary>
    /// Evaluates the test function for a specific <paramref name="point"/>.
    /// </summary>
    /// <param name="point">N-dimensional point for which the test function should be evaluated.</param>
    /// <returns>The result values of the function at the given point.</returns>
    public abstract double[] Evaluate(RealVector point);

  }
}
